;******************************************************************************************
;         Main.scm
; ******************************************************************************************
; Deze file bevat de main voor de hele logistieke achtergrond.
; Hij bevat de functie http-eval, die door de meta-level (java, http-deamon) wordt
; opgeroepen telkens er een request binnenkomt. Gelijktijdige requests worden gequeed
; (door het metalevel)
; de functie eval-http dispatched over de soort pagina, en roept dan de respectievelijke
; controller op. Elke request die binnenkomt bevat een tag, die aanduidt welke request
; ze bevat. Op basis van deze tag wordt gedispatched.
; Merken we dat de tag van de request gelijk is aan bv addboek, wordt de
; addboek controller
; opgeroepen. Dan wordt in addboek, de ganse input-stream omgezet in een bruikbaar formaat
; dat makkelijk te hanteren is in scheme. addboek roept de database aan, doet de nodige
; foutencontroles en stuurt uiteindelijk een resultaat in de vorm van een HTML-pagina
; terug naar de gebruiker


(load "objects.scm") ; bevat alle objectdefinities
(load "person-logica.scm") ; bevat alle logika achter het systeem
(load "book-logica.scm")
; deze file bevat de logistieke implementatie vor elke page.
(load "reservatie-logica.scm")
(load "security.scm")
(load "dbio.scm")
(load "logging.scm")

(display "funcs loaded")

(define path "htmls/")


(define (getpagetag str)
  (define index (indexof str ".html"))
  (if (= index -1)
      ""
      (substring str 5 index)))

(define (eval-http inputport outputport)
  (define test "")
  (define input (process-URL (do ()
                                 ((not (= (string-length test) 0)) test)
                                 (set! test (read-string inputport)))))
  (define pagetag (getpagetag input))
;  (display "<INPUT>")
;  (display input)
;  (display "</INPUT>")
  (display "pagetag is :")
  (display pagetag)
  (newline)
  ; evalueer de functie
  (define function-name (string->symbol (string-append pagetag "-scm")))
  (display function-name)
  (cond ((not (defined function-name))
         (no-url-scm input outputport))
        ((string=? pagetag "")
	 (display "START LOGIN")
	 (newline)
	 (apply (eval function-name) (list input outputport)))
	((string=? pagetag "handle-login")
	 (apply (eval function-name) (list input outputport)))
        (else
	  ; Security processing
          (if (not (check-login input))
             (output-html-page "error.html" '("%error") '("Wrong login") outputport)
             (apply (eval function-name) (list input outputport))))))

(define (splice template vars vals) ; vars lijst met vars, vals evenlange lijst met vals to replace var
  ; vervangt in de template alle vars door de overeenkomstige val.
  ; recursieve versie, wordt waarschijnlijk iteratief voor performantie en geheugen
  ; reden : HTML-page kan vrij groot worden.
  (define (replace-list var-list val-list)
    (cond ((null? var-list) (if (null? val-list) template "vecs not equally long in splice!"))
          ((null? val-list) '())
          ;          ((not (pair? var-list)) '())
          ;          ((not (pair? val-list)) '())
          (else (replace (replace-list (cdr var-list) (cdr val-list))
                         (car var-list)
                         (car val-list)))))
  (replace-list vars vals))


(define (output-html-page templatefile vars vals outputport)
  ; neemt een filename waar een template staat, spliced hierin alle variablen en
  ; stuurt deze terug naar de gebruiker
  ; let op no errochecking whatsoever, crashes if file doesn't exist, port fails, splice fails, ...
  ; (display "inside output-html")
  (define input (open-input-file (string-append path templatefile)))
  (if (null? input)
      (begin
        (writestring templatefile)
        (writestring " : unable to open template-file" outputport)
        (display "unable to open template-file"))
      (begin
        (define template (readstring input))
        (close input)
	(if (null? template)
            (begin
              (writestring templatefile)
              (writestring " : template-file contains no data" outputport)
              (display "unable to read from template"))
            (begin
              (display "splicing...")
              (define html-page (splice template vars vals))  ; replace vars with vals
              (set! html-page (replace html-page "%id" id))
              (set! html-page (replace html-page "%passwd" passwd))
              ;            (display "page generated, vars spliced ") (display html-page)
              (if (null? html-page)
                  (writestring "Something went terribly wrong in native-replace!!!" outputport)
                  (writestring html-page outputport)))))))
; send html-page to user


(define (startpage-scm input outputport)
 (cond
   ((string=? global_accesslevel "none")
    (output-html-page "error.html" '("%error") '("No authorized access. Contact us immediately !")
                       outputport))
   ((string=? global_accesslevel "igclient")
    (output-html-page "igclient.html" '("%id" "%passwd")
                        (list id passwd) outputport))
   ((string=? global_accesslevel "igadmin")
    (output-html-page "igadmin.html" '("%id" "%passwd")
                        (list id passwd) outputport))				   
   ((string=? global_accesslevel "reservatieclient")
    (output-html-page "reservatieclient.html" '("%id" "%passwd")
                        (list id passwd) outputport))
   (else (output-html-page "error.html" '("%error") '("No accesslevel set. Contact us immediately !")
                                                outputport))))


(display "starting http-eval")

;(load "dummy.scm") bevat alle dummy entries naar database, http, mail, ...  zijn ingevuld in meta-level





;------------------------------------
; elke html/scheme functie moet nen string weergeven
(define (-scm input outputport)
  (login-request-scm input outputport))

(define (no-url-scm input outputport)
  (output-html-page "bad-request.html" '() '() outputport))

(define (login-request-scm input outputport)
  (output-html-page "login-request.html" '() '() outputport))

(define (main-scm input outputport)
  (display "inside main")
  (output-html-page "main.html" '() '() outputport))

