; Securety-logika

; Globale variablen zodat een functie de id,passwd en accesslevel van de
;request die behandeld wordt kan opvragen.

(define id "")
(define passwd "")
(define global_accesslevel "")


; de id,passwd en accesslevel van een request worden gecached want de kans
;dat deze in de nabije toekomst terug binnenkomen is groot. Dit vermijd te
;frekwente db-accesses.

(define id-cache '())

; een entry in de id-cache krijgt een time-stamp mee.

(define (add-to-id-cache id passwd accesslevel)
  (set! id-cache (cons (list id passwd accesslevel (ticks)) id-cache)))
  
(define (cache-get-passwd id)
  (define result (assoc id id-cache))
  (if (null? result)
      #f
      (car result)))
      
(define (cache-get-accesslevel id)
  (define result (assoc id id-cache))
  (if (null? result)
      #f
      (cadr result)))

; Deze functie verwijdert te oude entry's in de cache. Ze wordt opgeroepen
;door de eval-idle functie.

(define (update-id-cache)
  (define current-time (ticks))
  (define (iter rest)
    (cond ((null? rest))
          ((< (cadr (cdr (cdr (car rest))))
	      (- current-time 9000000))
	   (iter (cdr rest)))
	  (else (cons (car rest) (iter (cdr rest))))))
  (set! id-cache (iter id-cache)))

; Deze functie behandelt de login vanaf de request-login pagina.
; Ze dispatcht op de accesslevel om de user het juiste menu te geven.

(define (handle-login-scm input outputport)
  (define login (get-var-value input "login"))
  (define password (get-var-value input "password"))
  (define query (create-person))
  (query 'set 'login login)
  (query 'set 'password password)
  ; een login-check gebeurd door een exacte match van een person met alleen
  ; login en passwd ingevuld op de databank.
  (let ((result (db-lookup-person query)))

    (cond ((or (= (string-length login) 0)
               (= (string-length password) 0))
	   (output-html-page "error.html" '("%error") (list "Wrong login")
                             outputport))
          ((null? result)
           (output-html-page "error.html" '("%error") (list "Wrong login")
                             outputport))
	  ((string? result)
	   (output-error result outputport))
	  ((not (= (length result) 1))
	   (output-error "Double match on login & passwd Contact us immediately !"
                             outputport))
          (else ; hier kunnen we uitgaan van een correcte lookup
	        (let* ((person (car result))
                       (personnumber (person 'get 'personnumber))
                       (accesslevel (person 'get 'accesslevel)))
		  (if (not (cache-get-passwd personnumber))
		      (add-to-id-cache personnumber password accesslevel))
		  (cond ((string=? accesslevel "none")
                         (output-html-page "error.html" '("%error") '("No authorized access. Contact us immediately !")
                                           outputport))
                        ((string=? accesslevel "igclient")
                         (output-html-page "igclient.html" '("%id" "%passwd")
                                           (list personnumber password) outputport))
	                ((string=? accesslevel "igadmin")
			 (send-notice "igadmin logged in")
                         (output-html-page "igadmin.html" '("%id" "%passwd")
                                           (list personnumber password) outputport))				   
                        ((string=? accesslevel "reservatieclient")
                         (output-html-page "reservatieclient.html" '("%id" "%passwd")
                                           (list personnumber password) outputport))
			((string=? accesslevel "reservatieadmin")
                         (output-html-page "reservatieadmin.html" '("%id" "%passwd")
                                           (list personnumber password) outputport))		   
                        (else (output-html-page "error.html" '("%error") '("No accesslevel set. Contact us immediately !")
                                                outputport))))))))

; de functie wordt opgeroepen bij het binnenkomen van een request ander dan
;de login-request. Hij controleert of login & passwd nog matchen en ook of
;de gevraagde functie behoort tot de accesslevel van de user (dmv de functie
;check-accesslevel)

(define (check-login input)
   (set! id (get-var-value input "id"))
   (set! passwd (get-var-value input "passwd"))
   (if (null? id) (set! id ""))
   (if (null? passwd) (set! passwd ""))
   (define cache-passwd-result (cache-get-passwd id))
   (define cache-accesslevel-result (cache-get-accesslevel id))
   (if cache-passwd-result
       (and (string=? cache-passwd-result passwd)
	    (check-accesslevel input id cache-accesslevel-result))
       (begin 
	    (define query (create-person))
	    (query 'set 'personnumber id)
	    (query 'set 'password passwd)
	    (define result (db-lookup-person query))
	    
             
         (cond ((null? result)
                 #f)
	       ((string? result)
	      	#f)
               ((not (= (length result) 1))
                #f)
               ((check-accesslevel input id ((car result) 'get 'accesslevel))
	        (add-to-id-cache id
		                 passwd
				 global_accesslevel)
		 #t)
	       (else #f)))))

; deze lijst bevat alle functie die mogen opgeroepen worden door een
;igclient-user

(define igclient-functions (list 'startpage
                                 'add-person
                                 'person-added
				 'lookup-person
				 'select-person
				 'lookup-person-results
				 'lookup-person-edit
				 'select-person-for-edit
				 'edit-person
				 'edited-person-info
				 'lookup-book
				 'select-book-for-lookup
				 'lookup-book-results
				 'lookup-person-for-loan
				 'person-display-for-loan
				 'select-person-for-loan
				 'select-book-for-loan
				 'get-book-for-loan
				 'loan-warranty
				 'loan-book
				 'select-book-bringback
				 'lookup-book-bringback
				 'bring-back-book
				 'borrowed-books
				 'make-reservation
				 'view-reservations
				 'view-reservation-week
                                 'add-reservation
				 'super-book-loan-show))

;idem maar voor de reservatieclient-functions

(define reservatieclient-functions (list 'startpage
                                         'make-reservation
                                         'view-reservations
					 'add-reservation))

(define reservatieadmin-functions (list  'startpage
                                         'make-reservation
                                         'view-reservations
					 'view-reservation-week
					 'add-reservation
					 'del-reservation
					 'delete-reservations
					 'confirm-delete))

;idem maar voor de igadmin functies

(define igadmin-functions (list 'startpage
                                'delete-person
                                'select-person-for-delete
				'confirm-person-delete
				'person-delete
				'draag-over
				'select-person-for-overdracht
				'update-person
				'draag-person-over
				'add-book
				'book-added
				'delete-book
				'select-book-for-delete
				'confirm-book-delete
				'book-delete
			        'lookup-book-edit
				'select-book-for-edit
				'edit-book
				'edited-book-info))


; deze functie controleert of de request behoort tot de toegelaten functies
; van de user volgens accesslevel

(define (check-accesslevel input id accesslevel)
  (define pagerequest (getpagetag input))
  (define accesslevel (cache-get-accesslevel id))
  (define (access-approved level request)
     (set! global_accesslevel level)
     (cond ((string=? level "igclient")
            (member (string->symbol pagerequest) igclient-functions))
	   ((string=? level "reservatieclient")
	    (member (string->symbol pagerequest) reservatieclient-functions))
	   ((string=? level "reservatieadmin")
	    (member (string->symbol pagerequest) reservatieadmin-functions))
	   ((string=? level "igadmin")
	    (member (string->symbol pagerequest) igadmin-functions))
	   (else
		 #f)))
  
  (if accesslevel
      (access-approved accesslevel pagerequest)
      (begin 
	    (define query (create-person))
	    (query 'set 'personnumber id)
            (define result (db-lookup-person query))

         (cond ((null? result)
                #f)
	       ((string? result)
	        #f)
               ((not (= (length result) 1))
                (result 'get 'accesslevel))
              (else (access-approved ((car result) 'get 'accesslevel) pagerequest))))))

; de eval-idle functie wordt opgeroepen door de evaluator nadat een bepaalde
;tijd geen request meer werden ontvangen.

(define (eval-idle)
   (display "IDLE CALLED")
   (update-id-cache))

